/*
 * Project: MoleCuilder
 * Description: creates and alters molecular systems
 * Copyright (C)  2010-2012 University of Bonn. All rights reserved.
 * 
 *
 *   This file is part of MoleCuilder.
 *
 *    MoleCuilder is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    MoleCuilder is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with MoleCuilder.  If not, see <http://www.gnu.org/licenses/>.
 */

/*
 * InvertMoleculesAction.cpp
 *
 *  Created on: Mar 12, 2012
 *      Author: heber
 */

// include config.h
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

//#include "CodePatterns/MemDebug.hpp"

#include "Descriptors/MoleculeDescriptor.hpp"
#include "Descriptors/MoleculeIdDescriptor.hpp"
#include "CodePatterns/Log.hpp"
#include "CodePatterns/Verbose.hpp"
#include "molecule.hpp"
#include "World.hpp"

#include <boost/foreach.hpp>
#include <iostream>
#include <string>

#include "InvertMoleculesAction.hpp"

using namespace MoleCuilder;

// and construct the stuff
#include "InvertMoleculesAction.def"
#include "Action_impl_pre.hpp"
/** =========== define the function ====================== */
ActionState::ptr SelectionInvertMoleculesAction::performCall() {
  World::getInstance().invertMoleculeSelection();
  LOG(0, World::getInstance().countSelectedMolecules() << " molecules selected.");

  return ActionState::ptr(new SelectionInvertMoleculesState(params));
}

ActionState::ptr SelectionInvertMoleculesAction::performUndo(ActionState::ptr _state) {
  //SelectionInvertMoleculesState *state = assert_cast<SelectionInvertMoleculesState*>(_state.get());

  World::getInstance().invertMoleculeSelection();
  LOG(0, World::getInstance().countSelectedMolecules() << " molecules selected.");

  return ActionState::ptr(_state);
}

ActionState::ptr SelectionInvertMoleculesAction::performRedo(ActionState::ptr _state){
  //SelectionInvertMoleculesState *state = assert_cast<SelectionInvertMoleculesState*>(_state.get());

  World::getInstance().invertMoleculeSelection();
  LOG(0, World::getInstance().countSelectedMolecules() << " molecules selected.");

  return ActionState::ptr(_state);
}

bool SelectionInvertMoleculesAction::canUndo() {
  return true;
}

bool SelectionInvertMoleculesAction::shouldUndo() {
  return true;
}
/** =========== end of function ====================== */
